package org.dive4elements.river.model;

import java.util.List;

import java.io.Serializable;
import java.math.BigDecimal;

import javax.persistence.Entity;
import javax.persistence.Id;
import javax.persistence.Table;
import javax.persistence.GeneratedValue;
import javax.persistence.Column;
import javax.persistence.SequenceGenerator;
import javax.persistence.GenerationType;
import javax.persistence.JoinColumn;
import javax.persistence.OneToOne;

import org.apache.log4j.Logger;

import org.hibernate.Session;
import org.hibernate.Query;

import org.dive4elements.river.backend.SessionHolder;


@Entity
@Table(name = "bed_height_epoch_values")
public class BedHeightEpochValue
implements   Serializable
{
    private static Logger logger =
        Logger.getLogger(BedHeightEpochValue.class);

    private Integer id;

    private BedHeightEpoch bedHeight;

    private BigDecimal station;
    private BigDecimal height;


    public BedHeightEpochValue() {
    }

    public BedHeightEpochValue(
        BedHeightEpoch bedHeight,
        BigDecimal station,
        BigDecimal height
    ) {
        this.bedHeight = bedHeight;
        this.station   = station;
        this.height    = height;
    }

    @Id
    @SequenceGenerator(
        name           = "SEQUENCE_BED_EPOCH_VALUE_ID_SEQ",
        sequenceName   = "BED_EPOCH_VALUES_ID_SEQ",
        allocationSize = 1)
    @GeneratedValue(
        strategy  = GenerationType.SEQUENCE,
        generator = "SEQUENCE_BED_EPOCH_VALUE_ID_SEQ")
    @Column(name = "id")
    public Integer getId() {
        return id;
    }

    public void setId(Integer id) {
        this.id = id;
    }

    @OneToOne
    @JoinColumn(name = "bed_height_epoch_id" )
    public BedHeightEpoch getBedHeight() {
        return bedHeight;
    }

    public void setBedHeight(BedHeightEpoch bedHeight) {
        this.bedHeight = bedHeight;
    }

    @Column(name = "station")
    public BigDecimal getStation() {
        return station;
    }

    public void setStation(BigDecimal station) {
        this.station = station;
    }

    @Column(name = "height")
    public BigDecimal getHeight() {
        return height;
    }

    public void setHeight(BigDecimal height) {
        this.height = height;
    }


    public static List<BedHeightEpochValue> getBedHeightEpochValues(
        BedHeightEpoch epoch,
        double kmLo,
        double kmHi
    ) {
        Session session = SessionHolder.HOLDER.get();

        Query query = session.createQuery(
            "from BedHeightEpochValue where bedHeight=:epoch " +
            "   and station >= :kmLo and station <= :kmHi");

        query.setParameter("epoch", epoch);
        query.setParameter("kmLo", new BigDecimal(kmLo));
        query.setParameter("kmHi", new BigDecimal(kmHi));

        return query.list();
    }
}
// vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8 :
